#import "SubViewRotationViewController.h"

#define kDegreesToRadian(x) (M_PI * (x) / 180)

@implementation SubViewRotationViewController
@synthesize pictureView, clippingView, degree;


- (void)viewWillAppear:(BOOL)animated
	{
	CGRect fullScreenRect = [[UIScreen mainScreen] bounds];
	degree = 1;
	
	// MOVING VIEW
	CGRect aPictureRect = CGRectMake(-100, -100, 500 , 500);
	UIView *aPictureView = [[UIView alloc] initWithFrame:aPictureRect];
	aPictureView.backgroundColor = [UIColor colorWithPatternImage:[UIImage imageNamed:@"picture.png"]];

	self.pictureView = aPictureView;
	[aPictureView release];
	
	// CLIPPING VIEW
	CGFloat clippingViewWidthFromCenter = 200;
	CGFloat clippingViewHeightFromCenter = 200;
	
	CGRect clippingRect = CGRectMake	(
							CGRectGetMidX(fullScreenRect) - clippingViewWidthFromCenter / 2,
							CGRectGetMidY(fullScreenRect) - clippingViewHeightFromCenter / 2,
							clippingViewWidthFromCenter,
							clippingViewHeightFromCenter
							);

	UIView *aClippingView = [[UIView alloc] initWithFrame:clippingRect];
	aClippingView.backgroundColor = [UIColor clearColor];
	aClippingView.clipsToBounds = YES;
	
	aClippingView.autoresizingMask =	(
							UIViewAutoresizingFlexibleLeftMargin |
							UIViewAutoresizingFlexibleRightMargin |
							UIViewAutoresizingFlexibleTopMargin |
							UIViewAutoresizingFlexibleBottomMargin
							);

	self.clippingView = aClippingView;
	[aClippingView release];
	
	// SUBVIEW HIERARCHY
	[self.view insertSubview:clippingView atIndex:0];
	[clippingView addSubview:pictureView];
	
	// START ROTATION
	[self rotate];
	
	[super viewWillAppear:animated];
	}

- (void)animationDidStop:(NSString *)animationID finished:(NSNumber *)finished context:(void *)context
	{
	if ((animationID == @"rotatePicture") && finished)
		{
		degree += 1;
		[self rotate];
		}
	}
	
- (void)rotate
	{
	[UIView beginAnimations:@"rotatePicture" context:NULL];
	[UIView setAnimationDelegate:self];
	[UIView setAnimationDidStopSelector:@selector(animationDidStop:finished:context:)];

	[UIView setAnimationDuration:0.05];
	[UIView setAnimationRepeatCount:1];
	[UIView setAnimationCurve:UIViewAnimationCurveLinear];
	
	CGAffineTransform rotateTransform = CGAffineTransformMakeRotation(kDegreesToRadian(degree));
	self.clippingView.transform = rotateTransform;

	[UIView commitAnimations];
	}



- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
	{
	return	(
		interfaceOrientation == UIInterfaceOrientationPortrait ||
		interfaceOrientation == UIInterfaceOrientationLandscapeLeft ||
		interfaceOrientation == UIInterfaceOrientationLandscapeRight
		);
	}
	
- (void)dealloc
	{
	[clippingView release];
	[pictureView release];
	[super dealloc];
	}

@end
